/* mndcombi.cpp  by Wolf Jung (C) 2007-2014.  Defines classes:
   mndAngle, mndCombi, mndPath

   These classes are part of Mandel 5.12, which is free software; you can
   redistribute and / or modify them under the terms of the GNU General
   Public License as published by the Free Software Foundation; either
   version 3, or (at your option) any later version. In short: there is
   no warranty of any kind; you must redistribute the source code as well.
*/

#include <cmath>
#include <new>
#include "mndynamo.h"

int mndAngle::normalize(qulonglong &n, qulonglong &d, int &k)
{  if (!d) return 0;
   n %= d; while (!(n & 1) && !(d & 1)) { n >>= 1; d >>= 1; }
   int p; qulonglong n0, n1 = n, d1 = d, np;
   k = 0; while (!(d1 & 1)) { k++; d1 >>= 1; if (n1 >= d1) n1 -= d1; }
   n0 = n1;
   for (p = 1; p <= 65 - k; p++) { twice(n1, d1); if (n1 == n0) break; }
   if (k + p > 64) return 0;
   np = 1LL << (p - 1); np--; np <<= 1; np++; //2^p - 1 for p <= 64
   n0 = np; d >>= k; n1 = d; if (n1 > n0) { n1 = n0; n0 = d; }
   while (1) { d1 = n0 % n1; if (!d1) break; n0 = n1; n1 = d1; } //gcd n1
   n /= d/n1; n *= np/n1; d = np << k;
   return p;
}

void mndAngle::twice(qulonglong &n, qulonglong &d)
{  if (n >= d) return;
   if (!(d & 1)) { d >>= 1; if (n >= d) n -= d; return; }
   qulonglong large = 1LL; large <<= 63; //avoid overflow:
   if (n < large) { n <<= 1; if (n >= d) n -= d; return; }
   n -= large; n <<= 1; large -= (d - large); n += large;
}

/*int mndAngle::conjugate32(qulonglong &n, qulonglong &d)
{  int j, k, p = normalize(n, d, k); if (!p || p > 32 || k) return 0;
   if (p == 2) n = 3LL - n; if (p <= 2) return 1;
   qulonglong n1, d1 = d << 1, np, na = n, nb = n + d;
   if (n & 1LL) n1 = na; else n1 = nb; //A is na/d1 < ... < nb/d1
   //compute preimages of n1/d1 on the same side as forward orbit
   for (j = 2; j <= p; j++)
   {  d1 <<= 1; na <<= 1; nb <<= 1;
      np = n; for (k = j; k < p; k++) twice(np, d); np <<= j;
      if (na < np && np < nb) { if (n1 < na || nb < n1) n1 += (d1 >> 1); }
      else { if (na < n1 && n1 < nb) n1 += (d1 >> 1); }
   }
   np = n; n1 = (n1 - n)/d; n = n1;
   for (j = 1; j <= p; j++) { twice(n1, d); if (n1 == np) break; }
   if (j >= p) return p;
   while (1) { k = p % j; if (!k) break; p = j; j = k; } //gcd j
   return j;
}//conjugate32*/

int mndAngle::conjugate(qulonglong &n, qulonglong &d)
{  int j, k, p = normalize(n, d, k); if (!p || k) return 0;
   if (p == 2) n = 3LL - n; if (p <= 2) return 1;
   qulonglong np, u, na, nb, n1; //A is na < np < nb
   if (n & 1LL) { u = 0LL; na = n >> 1; np = na + (d >> 1) + 1LL; nb = np; }
   else { u = 1LL; np = n >> 1; na = np; nb = np + (d >> 1) + 1LL; }
   //Compute preimages of  n/d , such that the preperiodic angle
   // (u + n/d)/2^j  is on the same side  A  or  B  as the periodic  np/d .
   for (j = 2; j <= p; j++)
   {  if (np & 1LL) { np >>= 1; np += d >> 1; np++; } else np >>= 1;
      //Check if the old preperiodic angle is  < n/d , label this by k = 1.
      //Only for p <= 32 :  if (d*u < ((1LL << (j - 1)) - 1LL)*n)
      k = 0; n1 = n >> (p + 1 - j); //first j - 1 digits of p digits
      if (u < n1) k = 1;
      else if (u == n1)
      {  n1 |= (((1LL << (p + 1 - j)) - 1LL) & n) << (j - 1); //swap words
         if (n < n1) k = 1;
      }
      if (k)  { if (na < np && np < nb) u += 1LL << (j - 1); }
      else  { if (np <= na || nb <= np) u += 1LL << (j - 1); }
   }
   n = u; //Preperiod digits of the preimage determine the conjugate angle.
   for (j = 1; j <= p; j++) { twice(np, d); if (np == u) break; }
   if (j >= p) return p;
   while (1) { k = p % j; if (!k) break; p = j; j = k; }
   return j; //gcd of former j and p is period of characteristic point.
}//conjugate64

qulonglong mndAngle::wake(int k, int r, qulonglong &n)
{  if (k <= 0 || k >= r || r > 64) return 0LL;
   qulonglong d = 1LL; int j, s = 0; n = 1LL;
   for (j = 1; j < r; j++)
   {  s -= k; if (s < 0) s += r; if (!s) return 0LL;
      if (s > r - k) n += d << j;
   }
   d <<= (r - 1); d--; d <<= 1; d++; //2^r - 1 for r <= 64
   return d;
}

ldouble mndAngle::radians(qulonglong n, qulonglong d)
{  if (n >= d) return 0.0;
   return 2.0L * PI * ((ldouble)(n)) / ((ldouble)(d));
}

int mndAngle::setAngle(qulonglong n, qulonglong d, int &k)
{  if (n >= d) return 0;
   N = n; D = d; P = normalize(N, D, K);
   k = K; return P;
}

/*
int mndAngle::realSpider(qulonglong n, qulonglong d, double &c)
{  if (n >= d) return 0;
   int j, k, p = normalize(n, d, k); if (k || !p) return 0;
//update this old implementation:
   long int num2 = denom - num, n = num; if (2*n > denom) n = denom - num;
   double z1 = 2.0*PI/double(denom), *z = new double[period+1];
   z[1] = cos(z1*n);
   for (j = 2; j <= 32; j++)
   {  if (j == 32) return -2;  //period > 30
      n *= 2; if (n > denom) n -= denom;
      if (num < n && n < num2) return j;  //z_j < z_1
      if (n == num) { period = j - 1; break; }
      //if (n == num2) from doubling
      z[j] = cos(z1*n);
   }
   int *side = new int[period]; z1 = cos(z1*0.5*num); z[period] = 0;
   for (j = 1; j < period; j++)
   { z[j] = 2.0*(z[j] - z1); if (z[j] < 0) side[j] = 1; else side[j] =0; }
   for (k = 1; k <= 50; k++)
   {  z1 = z[1];
      for (j = 1; j < period; j++)
      {  z[j] = z[j+1] - z1; if (z[j] < 0) z[j] = 0;
         z[j] = sqrt(z[j]); if (side[j]) z[j] = -z[j];
      }
   }
   c = z[1]; delete[] z; delete[] side; return 0;
   return 0;
}//*/

double mndAngle::lambda(qulonglong N, qulonglong D,
   double eps, int nmax) // = 1.0e-8, 100
{  eps *= 0.1; if (!N || N >= D) return 1.0;
   int j, k = 0; qulonglong s = 1ULL, N1 = N >> 1, N2 = N1 + (D >> 1);
   for (j = 1; j <= 63; j++)
   {  N <<= 1; if (N >= D) N -= D;
      if (N1 < N && N <= N2) s |= (1ULL << j);
      if (!N) { k = j; break; }
   }
   if (!k) return 3.0; if (k == 1) return 2.0;
   /* Now  k  is the preperiod and  s  is the kneading sequence,
      with bit  1 ~ c1 , 2 ~ c2 , 4 ~ c3 ...
      To determine a block of the Thurston matrix,  the path  [c, beta]
      is iterated.  A[j]  encodes the path  [c' , c'']  defined in step j
      as  2^16 * c' + c'' , with  beta ~ 0  and  c ~ k .
      X[j]  is a stack of elements not yet mapped.
      F[j] = 2^16 * a' + a''  means that path j is mapped to path a'
      and path a'', with  a'' = 0  when there is only one image.
      No double array is used to encode the matrix,  because there
      are only one or two nonzero entries per column.
   */
   int c1, c2, nf, na = 1, nx = 1, Na, Nx; int *A, *F, *X;
   if (k <= 22) Na = ((k*(k + 1)) >> 1) + 2; else Na = 255; Nx = Na;
   A = new int[Na+1]; F = new int[Na+1]; X = new int[Nx+1];
   A[1] = k; X[1] = 1;
   while (nx)
   {  nf = X[nx]; c1 = A[nf]; c2 = c1 & 0xFF; c1 >>= 16; nx--;
      if ( ((s >> (k - c1)) & 1) == ((s >> (k - c2)) & 1) ) //same side
      { c2--; if (c1) c1--; c1 <<= 16; c1 |= c2; c2 = 0; }
      else //covers 0, two images
      { c2--; if (c1) c1--; c1 <<= 16; c1 |= k; c2 <<= 16; c2 |= k; }
      for (j = 1; j <= na; j++) if (A[j] == c1)
      { F[nf] = j << 16; c1 = 0; break; }
      if (c1) { na++; A[na] = c1; F[nf] = na << 16; nx++; X[nx] = na; }
      if (na >= Na - 1) return (double)(na); //or increase Na
      if (!c2) continue;
      for (j = 1; j <= na; j++) if (A[j] == c2)
      { F[nf] |= j; c2 = 0; break; }
      if (c2) { na++; A[na] = c2; F[nf] |= na; nx++; X[nx] = na; }
      if (nx == Nx || na >= Na - 1) return (double)(nx); //or increase Na , Nx
   }
   delete A; delete X;
   /* Since the block is primitve,  the largest eigenvalue is obtained
      fast by iterating the matrix and computing the growth factor.
      In principle,  repeated squaring would be faster,  but it
      requires two full double arrays and more arithmetic operations.
   */
   qulonglong *V, *Vold, *Vtemp; double lambda = 42.0, lambdaold = 0.0;
   V = new qulonglong[na+1]; Vold = new qulonglong[na+1];
   Vold[0] = (qulonglong)(na); for (j = 1; j<= na; j++) Vold[j] = 1uLL;
   s = 1ULL << 63; nf = 0;
   for (k = 1; k <= nmax; k++)
   {  for (j = 0; j <= na; j++) V[j] = 0ULL;
      for (j = 1; j <= na; j++)
      {  nx = F[j] & 0xFF; if (nx) V[nx] += Vold[j];
         V[F[j] >> 16] += Vold[j];
      }
      for (j = 1; j<= na; j++) V[0] += V[j];
      if (V[0] & s) { Vold[0] >>= 2; for (j = 0; j <= na; j++) V[j] >>= 2; }
      lambda = ((double)(V[0])) / ((double)(Vold[0]));
      lambdaold -= lambda; if (lambdaold < 0) lambdaold = -lambdaold;
      if (lambdaold < eps) nf++; else nf = 0; if (nf & 4) break;
      lambdaold = lambda; Vtemp = V; V = Vold; Vold = Vtemp;
   }
   delete Vold; delete V; delete F; return lambda;
   /* nf is needed because there may be little change at the beginning.
      This happens, e.g., for  1/4  6351/524288  3067/524288 .
      Another problem is given by approximate 2-cycles, e.g., for
      87383/1048576  86431/1048576 .
   */
}

/* Assuming that  u1  and  u2  are the numerators of conjugate angles of
   period  k , and  w  is the numerator of an angle of period  q , this
   function returns the numerator of the tuned angle of period  qk .
   The optional argument  n  requests the closest larger angle of period
   n , which is used in  toAngles()  before calling  backAngles() .
*/
qulonglong mndAngle::truncatedTuning(int k, qulonglong u1, qulonglong u2,
   int q, qulonglong w, int n) // = 0
{  if (!n) n = q*k;
   /* The implementation assumes that all arguments are > 0 , the numerators
      are appropriate for periods  k  and  q , respectively,  q >= 2 , and
      k < n <= 64 . Moreover,  (q - 2)k < n <= qk . The formulas get
      involved by allowing  qk > 64 , which happens already for
      1-2-4-5-9-13-17-22-25  in  toAngles()  since  q = 3 , k = 22, n = 25 .
   */
   int i, j; qulonglong u, L, M = 0LL, N = 0LL;
   //N  is first  n  digits of the numerator of the qk-periodic tunded angle.
   //If the n-periodic angle is smaller than the qk-periodic one,  take  ++N .
   if (q*k <= 64)
   {  for (i = q - 1; i >= 0; i--) M |= (w & (1LL << i) ? u2 : u1) << i*k;
      if (n == q*k) return M;
      //Now  N  will be the first  n  digits.  Denote the last  qk - n  digits
      //by  L , then the truncated angle will be larger if  NL > LN .
      L = 1LL << (q*k - n); L--; L &= M; N = M >> (q*k - n);
      if (M > ((L << n) | N)) return N; else return ++N;
   }
   for (i = q - 1; i >= 0; i--)
   {  j = n + (i - q)*k; u = (w & (1LL << i) ? u2 : u1);
      if (j <= 0) { N |= u >> -j; break; }
      N |= u << j;
   }
   if (n == q*k) return N;
   for (i = q - 1; i >= 0; i--) M |= (w & (1LL << i) ? u2 : u1) << i*k;
   //Now  N  is the first  n  digits and  M  is the last 64 digits of the
   //tuned numerator.  Denote the last  qk - n  digits by  L , then the
   //truncated angle will be larger if  NL > LN .
   if (q*k <= 2*n) //L is not longer than N
   {  L = 1LL << (q*k - n - 1); L--; L <<= 1; L++; L &= M;
      u1 = N >> (2*n - q*k); //first  qk - n  digits
      if (u1 > L) return N; if (u1 < L) return ++N;
      u1 = N & ((1LL << (2*n - q*k)) - 1LL); //last  2n - qk  digits
      u2 = N >> (q*k - n); //first  2n - qk  digits
      if (u1 > u2) return N; if (u1 < u2) return ++N;
      u2 = 1LL << (q*k - n - 1); u2--; u2 <<= 1; u2++; u2 &= N; //last qk - n
      if (L > u2) return N; else return ++N;
   }
   //Finally,  L is longer than  N , so replace it with  ML , where  L
   //denotes the last  n  digits of the tuned numerator and  M  is
   //the middle  qk - 2n  digits,  then the truncated angle will be
   //larger if  NML > MLN . Note that  M  will be shorter than  N  and  L .
   L = 1LL << (n - 1); L--; L <<= 1; L++; L &= M; M = 0LL;
   for (i = q - 1; i >= 0; i--)
   {  j = i*k - n; u = (w & (1LL << i) ? u2 : u1);
      if (j <= 0) { M |= u >> -j; break; }
      M |= u << j;
   }
   M &= (1LL << (q*k - 2*n)) - 1LL;
   u1 = (M << (3*n - q*k)) | (L >> (q*k - 2*n));
   if (N > u1) return N; if (N < u1) return ++N;
   u2 = L & ((1LL << (q*k - 2*n)) - 1LL);
   if (M > u2) return N; if (M < u2) return ++N;
   if (L > N) return N; else return ++N;
}

/////////////////////////////////////////////////////////////////////

int mndCombi::phi(int n) //Euler totient for  n <= 120
{  if (n <= 0 || n > 120) return 0;
   int phi = 1;  if (!(n % 2)) n /= 2; while (!(n % 2)) { n /= 2; phi *= 2; }
   if (!(n % 3)) { n /= 3; phi *= 2; } while (!(n % 3)) { n /= 3; phi *= 3; }
   if (!(n % 5)) { n /= 5; phi *= 4; } while (!(n % 5)) { n /= 5; phi *= 5; }
   if (!(n % 7)) { n /= 7; phi *= 6; } while (!(n % 7)) { n /= 7; phi *= 7; }
   if (n > 1) phi *= (n - 1); //last  n  was prime if  1 < n < 121
   return phi;
}

/* rho(m)  is the smallest index  j > m , such that the  j-th  entry of the
   kneading sequence differs from the periodic continuation of the first
   m  entries,  or simpler,  such that it differs from the entry at  j - m .
   This implementation treats the kneading sequence as *-periodic,  thus
   rho(m) <= P  for  m < P , and it does not work for  m >= P .
*/
int mndCombi::rho(int m) const
{  if (m <= 0 || m >= P) return 0;
   int j; qulonglong u = K >> m; u ^= K;
   for (j = m + 1; j < P; j++) if ((u >> (P - j)) & 1LL) return j;
   return P;
}

int mndCombi::setKneading(qulonglong k)
{  qulonglong u = 1LL; if (k <= u) { P = 0; return 0; }
   K = k; for (P = 1; P <= 64; P++) { k >>= 1; if (!k) break; }
   int m = 1, j = 0; A = u;
   while (m < P) { j = m; m = rho(m); A |= u << (m - 1); }
   //K  is the upper sequence,  if it differs from  k  in the last entry:
   k = K >> j; if (!(k & u)) K |= u; else if (K & u) K--;
   return P;
}

int mndCombi::setAddress(qulonglong a)
{  K = 1LL; if (a <= K || !(a & 1LL)) { P = 0; return 0; }
   A = a; P = 1; int i, j, p;
   for (i = 2; i <= 64; i++)
   {  a >>= 1; if (!(a & 1LL)) continue;
      p = P; P = i; K <<= P - p;
      for (j = 1; j <= (P - 1)/p; j++) K |= K >> p;
      if (K & 1LL) K--; else K++;
   }
   return P;
}

int mndCombi::fromAngle(qulonglong n, qulonglong d)
{  K = 1LL; int i; P = mndAngle::normalize(n, d, i);
   if (i || !P) { P = 0; return 0; }
   qulonglong na = n >> 1, nb = na + (d >> 1) + 1; //na < n < nb is A
   for (i = 1; i < P; i++)
   { K <<= 1; mndAngle::twice(n, d); if (na < n && n < nb) K |= 1LL; }
   setKneading(K); return P; //resets last entry to upper sequence
}

int mndCombi::renorm() const
{  int m, r; if (!P) return 0;
   for (r = 2; r < P; r++)
   {  if (!((A >> (r - 1)) & 1LL)) continue;
      for (m = r + 1; m <= P; m++)
      {  if (!((A >> (m - 1)) & 1LL)) continue;
         if (m % r) { m = 0; break; }
      }
      if (m) return r;
   }
   return 1;
}

int mndCombi::failsBS() const
{  int m, r; if (!P) return -1;
   for (m = rho(1) + 1; m < P; m++)
   {  if ((A >> (m - 1)) & 1LL) continue; //m  in address,  thus not evil
      for (r = 2; r < m; r++) if (!(m % r) && rho(r) > m) { r = 0; break; }
      if (!r) continue; //not exact period  m
      r = rho(m) % m; if (!r) return m; // ~ r = m , so  m  is in  orb(r)
      while (r < P) { if (r == m) return m; r = rho(r); }
   }
   return 0;
}//*/

/*int mndCombi::failsBS() const
{  int m, r; if (!P) return -1;
   for (m = 3; m < P; m++)
   {  if ((A >> (m - 1)) & 1LL) continue; //m  in address,  thus not evil
      for (r = 1; r < m; r++) if (!(m % r) && rho(r) > m) { r = 0; break; }
      if (!r) continue; //not exact period  m
      r = rho(m) % m; if (!r) return m; //~ r = m , so  m  is in  orb(r)
      while (r < P) { if (r == m) return m; r = rho(r); }
   }
   return 0;
}//*/

int mndCombi::count() const //assumes the combinatorics is realized
{  int j, k = 1, n, q, N = 1; if (!P) return -1; //invalid
   //Determine the denominators for all periods in the internal address:
   for (n = 2; n <= P; n++)
   {  if (!((A >> (n - 1)) & 1LL)) continue; //not in the internal address
      q = n/k; j = n % k;
      if (j) { q += 2; while (j < k) { j = rho(j); if (j == k) q--; } }
      N *= phi(q); k = n;
   }
   return N;
}

int mndCombi::toAngles(qulonglong &n1, qulonglong &n2, qulonglong &d) const
{  if (!P) return 1000; //invalid
   int j, k = 1, n, q; n1 = 0LL; n2 = 1LL; d = 1LL;
   //Determine characteristic angles for all periods in the internal address:
   for (n = 2; n <= P; n++)
   {  if (!((A >> (n - 1)) & 1LL)) continue; //not in the internal address
      //Determine the denominator of the sublimb  1/q :
      q = n/k; j = n % k;
      if (j) { q += 2; while (j < k) { j = rho(j); if (j == k) q--; } }
      //Restrict the old characteristic wake to  n-periodic angles:
      d = n1; n1 = mndAngle::truncatedTuning(k, d, n2, q, 1LL, n);
      n2 = mndAngle::truncatedTuning(k, d, n2, q, 2LL, n);
      d = 1LL << (n - 1); d--; d <<= 1; d++; //2^n - 1 for n <= 64
      if (!(n % k)) { k = n; continue; } //satellite bifurcation
      k = n; n2--; // n1/d  is larger and  n2/d  is smaller than  qk-angles.
      if (n2 - n1 == 1LL) continue; //always admissible
#ifdef COUNT_EARLY_RETURNS
      qulonglong u, u1 = n1, u2 = n2;
#endif
      //Iterate the old wake backwards along the new kneading sequence:
      if (backAngles(n, n1, n2, d)) return n; //Error, thus not admissible
#ifdef COUNT_EARLY_RETURNS
      if (n == P) //count the returns to previous wake
      {  q = 0; u = n1;
         for (j = 1; j < n; j++)
         { mndAngle::twice(u, d); if (u1 <= u && u <= u2) q++; }
         return -q;
      }
#endif
   }
   return 0;
}

int mndCombi::backAngles(int n,
   qulonglong &n1, qulonglong &n2, qulonglong d) const
{  int j; qulonglong N1, N2, N3, N4;
   //The preimages  [N1, N2]  and  [N3, N4]  of the reduced characteristic
   //wake  [n1, n2]  will be pulled back following the kneading sequence.
   N1 = (n1 + 1LL) >> 1; N2 = n2 >> 1;
   N4 = (d >> 1) + 1LL; N3 = (n1 >> 1) + N4; N4 += (n2 - 1LL) >> 1;
   for (j = n - 1; j > 0; j--)
   {  //When preimages belong to the characteristic wake,  try subdivision:
      if (n1 <= N1 && N1 <= n2)
      {  if (N1 > N4) { N1 = N3; N4 = N2; }
         N2 = n2; n2 = N1 - 1LL; //first subinterval
         if (n1 < n2 && !backAngles(n, n1, n2, d)) return 0;
         n1 = N4 + 1LL; n2 = N2; //second subinterval
         if (n1 < n2) return backAngles(n, n1, n2, d); else return 1;
      }
      //The preimages of the intervals are shrunk to  1/2  at least,
      //or more depending on a numerator being even or odd:
      if ((K >> (P - j)) & 1LL) //A in kneading sequence
      {  if (N1 > n2) { N1++; N1 >>= 1; N2 >>= 1; }
         else { N1 >>= 1; N1++; N1 += d >> 1; N2++; N2 >>= 1; N2 += d >> 1; }
         if (N3 > n2) { N3++; N3 >>= 1; N4 >>= 1; }
         else { N3 >>= 1; N3++; N3 += d >> 1; N4++; N4 >>= 1; N4 += d >> 1; }
      }
      else //B in kneading sequence
      {  if (N2 < n1) { N1++; N1 >>= 1; N2 >>= 1; }
         else { N1 >>= 1; N1++; N1 += d >> 1; N2++; N2 >>= 1; N2 += d >> 1; }
         if (N4 < n1) { N3++; N3 >>= 1; N4 >>= 1; }
         else { N3 >>= 1; N3++; N3 += d >> 1; N4++; N4 >>= 1; N4 += d >> 1; }
      }
      if (N1 > N2 || N3 > N4) return 1;
   }
   //Now  N1 = N2  and  N3 = N4  are the new characteristic angles times d.
   if (N1 > N3) { n2 = N1; n1 = N3; } else { n1 = N1; n2 = N3; }
   return 0;
}//backAngles

/* Computing external angles from an internal address or kneading sequence

Suppose the external angles of  1-...-k  are known already and those of
1-...-k-n  are to be determined.  First compute the denominator  q  from
the Bruin-Schleicher formula:  q = n/k  when  k | n  and
q = [n/k] + 1  or  [n/k] + 2  if  n in orb_rho(n % k)  or not,  respectively.
The implementation is choosing the  1/q-sublimb,  but it could be extended
to work with any angled internal address.

The angles  t1  and  t2  of the 1/q-sublimb are found with Douady tuning.
If  n = qk , we are done.  If not,  this  qk-periodic sector is iterated
back along the kneading sequence,  giving a sequence of strips around the
desired  n-periodic rays.  The strip around  z = 0  is defined by the
intervals  [T1, T2]  and  [T3, T4] . Its complementary sectors are contained
in the pieces A and B.  So for  j = n-1 ... j = 1 , iterating back means:
* If the  j-th  entry is  A , then  T1  and  T2  are mapped with
  T -> T/2  if  T1 > t2  and with  T -> (T + 1)/2  if  T2 < t1 .
* If the  j-th  entry is  B , it is vice versa.
* [T3, T4]  is iterated back with the same formulas.
It may happen that a preimage of the strip is contained in the sector,  so
t1 < T1 < T2 < T3 < T4 < t2  (after interchanging  Ti  if necessary). Then
we do not know which of its two preimages will be on the correct side,
and the algorithm is branching to try both possibilities:
* Assuming the characteristic angles of period  n  are in  (t1, T1) ,
  the preimages in  (0, 1/2)  will be on side  A  and those in  (1/2, 1)
  will be on side  B .
* Assuming the characteristic angles of period  n  are in  (T4, t2) ,  the
  other preimage of the strip will be the correct one.

Each branch turns up a narrow strip at  j = 1 , which should contain the
desired characteristic rays of period  n . These things can go wrong:
*  [T1, T2]  and  [T3, T4]  do not belong to the reduced wake assumed.
*  [T1, T2]  and  [T3, T4]  need not contain  n-periodic angles.
The second error happens only when the algorithm was restarted at  j = n
with the reduced wake,  and then the branch is stopped before the
first error will happen.

Proposition:
1) If nothing goes wrong in a branch of this algorithm,  it will give the
   correct angles.  In particular, this branch is the only one going through,
   and the internal address is admissible.
2) If all branches are failing,  the internal address will not be admissible.

Proof:
1) The angles will have the desired kneading sequence and belong to the
   1/q-sublimb of period k .
2) If the address is admissible,  consider the dynamics for the corresponding
   center.  There is a strip around the critical value,  which is iterated
   2:1  to the characteristic wake in  n  iterations.  The corresponding
   image strips are contained in larger strips,  which are preimages of the
   characteristic wake of period  qk : iterating it backwards is well-defined,
   since otherwise a large strip would have bifurcated somewhere between
   the centers of periods  qk  and  n . This does not happen for preimages
   of order  <= n  of the strip defined by the precharacteristic points,
   since these do not contain  0 , because there is a stable branch point
   separating the critical value from the other marked points in the  qk-wake.
   So,  if the algorithm is branching,  there will be a branch assuming the
   correct location of the  n-wake between the preimages of the  qk-wake,
   and all of the strips are well-defined strips around the  n-periodic rays.
For the other branches,  or if the address is not admissible,  we may still
assume that the intervals  [T1, T2]  and  [T3, T4]  are describing unlinked
strips,  because this will be the case for the center of period  qk .
The implementation is simplified by restarting the iteration with the
smaller intervals  (t1, T1)  or  (T4, t2)  when the algorithm is branching.
Considering these subsets is possible although they are not wakes.

Question:
Can the critical orbit return early more than once for an admissible
combinatorics?  If not, the recursive subdivision could be simplified.
There are only single early returns for periods  n <= 33 , and the first
example is given by  1-2-6-7-13-14 .

Implementation with  n-periodic angles:
t1  and  t2  are replaced with  n-periodic angles  n1/d > t1  and  n2/d < t2 .
Choosing the closest possible angles is quite involved,  since for large  qk
we cannot take  n2 = [d*t2]  because floating-point arithmetics is inaccurate.
The rest of the algorithm is simplified for  n <= 64, because the denominators
are not increased under iterating backwards (half-integer numerators are
rounded to the appropriate side),  and it is easy to see if the strip does
not contain an  n-periodic angle.  While a preimage of the  qk-periodic
sector would never touch its boundary,  ni/d  may have a period dividing  n .
Examples:  for the non-admissible  1-2-4-5-6 , both  n1/d  and  n2/d  are
3-periodic instead of  6-periodic.  For the admissible  1-3-6-7-10-12 ,
n1/d  is  12-periodic but  n2/d  is  4-periodic.
*/

/////////////////////////////////////////////////////////////////////

//use  mndynamics::root()  instead,  when that will have  ldouble .
void root(ldouble x, ldouble y, ldouble &u, ldouble &v)
{  v = sqrt(x*x + y*y);
   if (x > 0.0L) { u = sqrt(0.5L*(v + x)); v = 0.5L*y/u; return; }
   if (x < 0.0L)
   { v = sqrt(0.5L*(v - x)); if (y < 0.0L) v = -v; u = 0.5L*y/v; return; }
   if (y >= 0.0L) { u = sqrt(0.5L*y); v = u; return; }
   u = sqrt(-0.5L*y); v = -u;
}

const ldouble mndPath::eps = 1.0e-12L; //square of admissible distance to 0

/* For the Spider Algorithm,  check if  z  is between the line segment
   and the hyperbola segment through  a  and  b.
*/
int mndPath::twisted(ldouble a1, ldouble a2, ldouble b1, ldouble b2,
   ldouble z1, ldouble z2)
{  //return twisted(a1, a2, b1, b2, z1, z2, z1, z2);
   double t; //switch such that  a < b  in circular order:
   if (a2*b1 > a1*b2) { t = a1; a1 = b1; b1 = t; t = a2; a2 = b2; b2 = t; }
   if (a1*z2 < a2*z1 || b1*z2 > b2*z1) return 0; //z not in sector
   if ((z2 - a2)*(b1 - a1) < (z1 - a1)*(b2 - a2)) return 0; //above line
   t = a1*a1 - a2*a2; a2 *= 2.0L*a1; a1 = t;
   t = b1*b1 - b2*b2; b2 *= 2.0L*b1; b1 = t;
   t = z1*z1 - z2*z2; z2 *= 2.0L*z1; z1 = t;
   if ((z2 - a2)*(b1 - a1) < (z1 - a1)*(b2 - a2)) return 1; //above hyperbola
   else return 0;
}

/* Hyperbolas are mapped to line segments and line segments to parabolas.
   Check if the difference of parabola segments is enclosing 0.  Then
   the synchronous deformation of hyperbolas to line segments contains an
   intersection,  where the homotopy type of the lift has changed.  One
   curve is from  a  to  u  and the other one from  b  to  v  initially.
   Both implementations get vague for an approximately linear difference.
   The twist of 4/8191 with M = 2 at c_1 after 12 steps is not recognized.
*/
int mndPath::twisted(ldouble a1, ldouble a2, ldouble u1, ldouble u2,
   ldouble b1, ldouble b2, ldouble v1, ldouble v2)
{  double w1, w2, s, t = (a1*u2 > a2*u1 ? 1.0L : -1.0L); //opposite sides:
   if ((t*(a1*b2 - a2*b1) < -eps || t*(b1*u2 - b2*u1) < -eps)
    && (t*(a1*v2 - a2*v1) < -eps || t*(v1*u2 - v2*u1) < -eps)) return 0;
   u1 -= a1; u2 -= a2; v1 -= b1; v2 -= b2;
   //Check if parabola  A + Bt + Ct^2 is enclosing 0,  with
   //A = a^2 - b^2 , B = 2(au - bv) , C = u^2 - v^2 . a = A , w = B, u = C .
   w1 = 2.0L*(a1*u1 - a2*u2 - b1*v1 + b2*v2);
   w2 = 2.0L*(a1*u2 + a2*u1 - b1*v2 - b2*v1);
   t = a1*a1 - a2*a2 - b1*b1 + b2*b2; a2 = 2.0L*(a1*a2 - b1*b2); a1 = t;
   t = u1*u1 - u2*u2 - v1*v1 + v2*v2; u2 = 2.0L*(u1*u2 - v1*v2); u1 = t;
   if (2.0L*(w1*w1 + w2*w2 + u1*u1 + u2*u2) < a1*a1 + a2*a2) return 0;
   //Deformation from  t^2  to  t  is  t^2 <= s <= t .
   v1 = w1*u2 - w2*u1; if (v1 > -eps*eps && v1 < eps*eps) return 1; //unsafe
   t = (u1*a2 - u2*a1)/v1; s = (w2*a1 - w1*a2)/v1;
   if (t*t <= s && s <= t && t <= 1.0L) return 1;
   else return 0;
}//*/
/*int mndPath::twisted(ldouble a1, ldouble a2, ldouble u1, ldouble u2,
   ldouble b1, ldouble b2, ldouble v1, ldouble v2)
{  double w1, w2, t = a1*u2 - a2*u1; //opposite sides:
   if ((t*(a1*b2 - a2*b1) < 0.0L || t*(b1*u2 - b2*u1) < 0.0L)
    && (t*(a1*v2 - a2*v1) < 0.0L || t*(v1*u2 - v2*u1) < 0.0L)) return 0;
   u1 -= a1; u2 -= a2; v1 -= b1; v2 -= b2;
   w1 = a1*u1 - a2*u2 + b1*v1 + b2*v2; w2 = a1*u2 + a2*u1 - b1*v2 - b2*v1;
   t = u1*u1 - u2*u2 - v1*v1 + v2*v2; u2 = u1*u2 - v1*v2; u1 = 0.5L*t;
   v1 = a1*a1 - a2*a2 - b1*b1 + b2*b2; v2 = 2.0L*(b1*b2 - a1*a2);
   b1 = v1*u1 - v2*u2; b2 = v1*u2 + v2*u1;
   if (b2 < eps && b2 > -eps) return 0;
   //check if parabola is intersecting the ray perpendicular to the line
   t = -a2/b2; if (t > 0.0L && t < 1.0L && a1 + b1*t <= 0.0L) return 1;
   else return 0;
}//*/

/* The initial segment  0 <= t <= 1  in moduli space is discretized by
   m = M(1 - t) , so  c_j(0) ~ z[j-1][M-1]  and  c_j(1) = z[j-1][0] .
   At time 1 these points are initialized by the angles,  and at time 0
   by  c_{j+1}(0) = (c_j(1))^2 + c  with  c = c_1(0) = -exp(i2PI theta_p)
   in the periodic case and  c_1(0) = c_1(1)    in the preperiodic case.
   In the latter case,  c_{k+p} = -c_k  is not satisifed for  t < 1 .
   There is an additional exponential rescaling unless the radius is 1.
   In principle the iteration of the path is equivalent for all radii,
   but introducing the radius serves three purposes:  it is probably
   making hyperbolas safer to straighten in the beginning steps, it
   makes the visualization more instructive,  and it is required for
   the generalization to matings.
*/
int mndPath::angleInit(qulonglong N, qulonglong D, int newM, ldouble logR)
{  if (x) { delete[] x; delete[] y; }
   if (P > 0) { P = 0; delete[] X; delete[] Y; X = 0; Y = 0; }
   S = 0; M = newM; if (M < 2 || M > Mmax) M = Mmax;
   p = mndAngle::normalize(N, D, k);
   if (!p || (p == 1 && !k)) { M = 0; x = 0; y = 0; return 0; }
   x = new(std::nothrow) ldouble[k+p][Mmax];
   y = new(std::nothrow) ldouble[k+p][Mmax];
   if (!x || !y) { M = 0; delete[] x; delete[] y; x = 0; y = 0; return 0; }
   int j, m; ldouble a, u, v;
   for (j = 0; j < k + p; j++)
   {  v = mndAngle::radians(N, D); x[j][0] = cos(v); y[j][0] = sin(v);
      mndAngle::twice(N, D);
   }
   if (k)
   {  for (m = 1; m < M; m++)
      {  x[0][m] = x[0][0]; y[0][m] = y[0][0];
         v = ((ldouble)(m))/((ldouble)(M)); u = v*x[0][0]; v *= y[0][0];
         for (j = 1; j < k + p; j++)
         { x[j][m] = x[j][0] + u; y[j][m] = y[j][0] + v; }
      }
   }
   else
   {  for (m = 1; m < M; m++)
      {  a = ((ldouble)(m))/((ldouble)(M)); u = a*x[p-1][0]; v = a*y[p-1][0];
         x[0][m] = (1.0L - a)*x[0][0] - u; y[0][m] = (1.0L - a)*y[0][0] - v;
         for (j = 1; j <= p - 2; j++)
         { x[j][m] = x[j][0] - u; y[j][m] = y[j][0] - v;}
      }
      for (m = 0; m < M; m++) { x[p-1][m] = 0.0L; y[p-1][m] = 0.0L; }
   }
   for (m = 0; m < M; m++)
   {  u = exp(logR*exp(logTwo*((ldouble)(m))/((ldouble)(M)) ));
      for (j = 0; j < k + p; j++) { x[j][m] *= u; y[j][m] *= u; }
   }
   return M;
}

/* The given path  Z[0...M]  is from  c  to  c_1 = z_0 , which is mapped
   to  c  by  f^n . Since  z[j][m]  is labeled with decreasing index  m < M ,
   this means  Z[0] = c ~ z[0][M-1]  and  Z[M] ~ c1 = z[0][0] .
*/
int mndPath::shiftInit(int n, int m, ldouble *xx, ldouble *yy)
{  if (x) { delete[] x; delete[] y; }
   if (X) { delete[] X; delete[] Y; X = 0; Y = 0; }
   S = 0; P = 0; M = m; p = n; //k = 0 below
   if (n <= 2 || n > 64 || M < 2 || M > 16*Mmax)
   { M = 0; x = 0; y = 0; return 0; }
   ldouble a = xx[0], b = yy[0], u, v;
   while (M > Mmax)
   { M /= 2; for (k = 0; k < M; k++) { xx[k] = xx[2*k]; yy[k] = yy[2*k]; } }
   x = new(std::nothrow) ldouble[p][Mmax];
   y = new(std::nothrow) ldouble[p][Mmax];
   if (!x || !y) { M = 0; delete[] x; delete[] y; x = 0; y = 0; return 0; }
   x[0][1] = xx[m]; y[0][1] = yy[m]; //not M
   for (m = 0; m < M; m++) { x[p-1][m] = 0.0L; y[p-1][m] = 0.0L; }
   //iterate forward and backwards to find exact preimages
   for (k = 1; k <= p - 2; k++) 
   {  x[k][1] = x[k-1][1]*x[k-1][1] - y[k-1][1]*y[k-1][1] + a;
      y[k][1] = 2.0L*x[k-1][1]*y[k-1][1] + b;
   }
   for (k = p - 2; k >= 0; k--)
   {  root(x[k+1][1] - a, y[k+1][1] - b, u, v);
      if (u*x[k][1] + v*y[k][1] > 0.0L) { x[k][0] = u; y[k][0] = v; }
      else { x[k][0] = -u; y[k][0] = -v; }
   }
   for (m = 1; m < M; m++) { x[0][m] = xx[M-m]; y [0][m] = yy[M-m]; }
   for (k = 1; k <= p - 2; k++) for (m = 1; m < M; m++)
   { x[k][m] = x[k][0]; y[k][m] = y[k][0]; }
   k = 0; return M;
}

/* The postcritical points are labeled and iterated as follows:
   The critical value is  c_1 = z[0]  and the last point  c_{k+p} = z[k+p-1]
   is  0  in the periodic case or  -c_k = -z[k-1]  in the preperiodic case.
*/
int mndPath::pathStep(ldouble &a, ldouble &b)
{  if (!M || P) return -1; if (S > 0) return spiderStep(a, b); else S--;
   int i, j, l, m; ldouble u, v;
   for (m = M - 1; m >= 0; m--) //pull back
   {  i = m + 1; if (i == M) i = 0; a = x[0][m]; b = y[0][m];
      for (j = 0; j <= k + p - 2; j++)
      {  root(x[j+1][m] - a, y[j+1][m] - b, u, v);
         if (u*u + v*v < eps) return -m - 1;
         if (u*x[j][i] + v*y[j][i] >= 0.0L) { x[j][m] = u; y[j][m] = v; }
         else { x[j][m] = -u; y[j][m] = -v; }
      }
      if (k) { x[k+p-1][m] = -x[k-1][m]; y[k+p-1][m] = -y[k-1][m]; }
   }
   a = x[0][0]; b = y[0][0];
   for (m = M - 1; m >= 0; m--) //check for twists
   {  i = m + 1; if (i == M) i = 0;
      for (j = 1; j < k + p; j++)
      {  if (!k && j == p - 1) continue;
         for (l = 0; l < j; l++) if (twisted(x[j][m], y[j][m],
            x[j][i], y[j][i], x[l][m], y[l][m], x[l][i], y[l][i]))
         return m + 1;
      }
   }
   /*cout << endl; for (j = 0; j <= k + p - 2; j++)
      cout << x[j][0] << "    " << y[j][0] << endl;//*/
   /*if (S != -5) return 0; cout.precision(15); cout << endl;
   for (m = 0; m < M; m += 50) for (j = 6; j <= 7; j++)
      cout << x[j][m] << "    " << y[j][m] << endl;//*/
   /*if (S != -5) return 0; cout.precision(15); cout << endl;
   for (m = 0; m < M; m += 50) for (j = 6; j <= 7; j++)
      cout << x[j][m]*x[j][m] - y[j][m]*y[j][m] << "    "
           << 2.0L*x[j][m]*y[j][m] << endl;//*/
   /*if (S != -5) return 0; cout.precision(15); cout << endl;
   for (m = 0; m < M; m += 50)
      cout << x[7][m] - x[6][m] << "    " << y[7][m] - y[6][m] << endl;//*/
   return 0;
}

/* Mating is done with quadratic rational functions of the form
   [ (c-1)z^2 + c(C-1) ] / [ (C-1) + C(c-1)z^2 ] , which are fixing 1.
   c_j = z[j-1]  is the orbit of 0 and  1 / C_j = 1 / Z[j-1]  is the orbit
   of infinity.  For simplicity,  the initialization is done by the functions
   from the polynomial algorithm.  Although the radius is smaller than in the
   polynomial case,  this will not work for large periods.
*/
/*int mndPath::mateInit(qulonglong n, qulonglong d, qulonglong N, qulonglong D)
{  if (!angleInit(N, D, -9.210340371976183604L)) return 0;
   //Radius 0.0001 . Sets S = 0, P = 0, M = Mmax
   X = new(std::nothrow) ldouble[k+p][Mmax];
   Y = new(std::nothrow) ldouble[k+p][Mmax];
   if (!X || !Y)
   {  M = 0; delete[] x; delete[] y; x = 0; y = 0;
      delete[] X; delete[] Y; X = 0; Y = 0; return 0;
   }
   int j, m; for (j = 0; j < k + p; j++) for (m = 0; m < M; m++)
   { X[j][m] = x[j][m]; Y[j][m] = y[j][m]; }
   K = k; P = -p; //P < 0 so angleInit() does not delete X, Y
   if (!angleInit(n, d, -9.210340371976183604L))
   { M = 0; delete[] X; delete[] Y; X = 0; Y = 0; return 0; }
   P = -P; return M;
}//*/

/* The present implementation does not check for twists.  In future versions,
   consecutive points will define circle segments,  whoose square-root
   must be checked for twists.
   Matings will be available in  Mandel 6.0  with "slow" coloring.
*/
/*int mndPath::mateStep(ldouble &a1, ldouble &a2, ldouble &b1, ldouble &b2)
{  if (!M || !P) return -1;
   
   a1 = x[0][0]; a2 = y[0][0]; b1 = X[0][0]; b2 = Y[0][0];
   return 0;
}//*/

/* Each leg has  M  segments consisting of  S  smaller segments.  M  is
   initially  Mmax/2  and it is increased by 1 at each step.  The outer
   segments are initialized identically in the range from  10^5  to  10^10 .
   The  spiderStep()  is increasing  M  and pulling back for  m < M .  M
   is not increased beyond  Mmax , so the legs will be pulled in eventually.
   In this tentative implementation,  there is no refining and no pruning.
   The points are indexed as  z[j*S+s][m] .
*/
int mndPath::spiderInit(qulonglong N, qulonglong D)
{  if (x) { delete[] x; delete[] y; }
   if (X) { delete[] X; delete[] Y; X = 0; Y = 0; }
   S = 10; P = 0; M = Mmax/2; int j, s, m; double u, v;
   p = mndAngle::normalize(N, D, k);
   if (!p || (p == 1 && !k)) { M = 0; x = 0; y = 0; return 0; }
   x = new(std::nothrow) ldouble[(k+p)*S][Mmax];
   y = new(std::nothrow) ldouble[(k+p)*S][Mmax];
   if (!x || !y) { M = 0; delete[] x; delete[] y; x = 0; y = 0; return 0; }
   //exponential scaling such that  M++  corresponds to a square-root:
   for (m = 0; m <= M; m++) for (s = 0; s < S; s++)
      y[(k+p-1)*S+s][m] = exp(11.5L*exp(logTwo*(s/(ldouble(S)) + m - M)));
   for (j = 0; j < k + p - 1; j++)
   {  v = mndAngle::radians(N, D); u = cos(v); v = sin(v);
      mndAngle::twice(N, D);
      for (m = 0; m <= M; m++) for (s = 0; s < S; s++)
      { x[j*S+s][m] = u*y[(k+p-1)*S+s][m]; y[j*S+s][m] = v*y[(k+p-1)*S+s][m];}
   }
   if (k) for (m = 0; m <= M; m++) for (s = 0; s < S; s++)
   {  x[(k+p-1)*S+s][m] = -x[(k-1)*S+s][m];
      y[(k+p-1)*S+s][m] = -y[(k-1)*S+s][m];
   }
   else
   {  v = mndAngle::radians(N, D); u = cos(v); v = sin(v);
      ldouble t = y[(k+p-1)*S][0];
      for (m = 0; m <= M; m++) for (s = 0; s < S; s++)
      {  x[(k+p-1)*S+s][m] = u*(y[(k+p-1)*S+s][m] - t);
         y[(k+p-1)*S+s][m] = v*(y[(k+p-1)*S+s][m] - t);
      }
   }
   for (m = M + 1; m < Mmax; m++) for (j = 0; j < (k + p)*S; j++)
   { x[j][m] = x[j][M]; y[j][m] = y[j][M]; } //copy outer segments
   return M;
}

int mndPath::spiderStep(ldouble &a, ldouble &b)
{  int j, l, m, s; if (!M) return -1; if (S <= 0) return -1;
   ldouble A = x[0][0], B = y[0][0], u, v, xr, yr, u0 = 0.0L, v0 = 0.0L;
   if (M < Mmax) M++; //outer segment added.  Pulled back in the next step.
   for (m = M - 1; m >= 0; m--) for (s = S - 1; s >= 0; s--) //pull back
   {  if (!k) root(x[s][m] - A, y[s][m]- B, u0, v0);
      for (j = 0; j < k + p; j++)
      {  if (j == k + p - 1 && k) break;
         if (s < S - 1) { xr = x[j*S+s+1][m]; yr = y[j*S+s+1][m]; }
         else if (m < M - 1) { xr = x[j*S][m+1]; yr = y[j*S][m+1]; }
         else { xr = x[j*S+s][m]; yr = y[j*S+s][m]; }
         if (j == p - 1 && ! k) { u = u0; v = v0; }
         else root(x[(j+1)*S+s][m] - A, y[(j+1)*S+s][m] - B, u, v);
         if (u*u + v*v < eps && (k || j < p -1)) return -1;
         if (u*xr + v*yr > 0.0L) { x[j*S+s][m] = u; y[j*S+s][m] = v; }
         else { x[j*S+s][m] = -u; y[j*S+s][m] = -v; }
      }
      if (k)
      {  x[(k+p-1)*S+s][m] = - x[(k-1)*S+s][m];
         y[(k+p-1)*S+s][m] = - y[(k-1)*S+s][m];
      }
   }
   a = x[0][0]; b = y[0][0]; //check for twists
   for (j = 0; j < k + p; j++) for (m = 0; m <= M - 2; m++)
   {  if (x[j*S][m]*x[j*S][m] + y[j*S][m]*y[j*S][m] > 4.0L) break;
      for (l = 0; l < k + p; l++)
      {  if (l == j || (!k && l == p - 1)) continue;
         for (s = 1; s < S; s++) if (twisted(x[j*S+s-1][m], y[j*S+s-1][m],
             x[j*S+s][m], y[j*S+s][m], x[l*S][0], y[l*S][0])) return j + 1;
         if (twisted(x[j*S+S-1][m], y[j*S+S-1][m],
             x[j*S][m+1], y[j*S][m+1], x[l*S][0], y[l*S][0])) return j + 1;
      }
   }
   return 0;
}
