/* mndynamo.cpp  by Wolf Jung (C) 2007-2014.  Defines classes:
   mndynamics, mndsiegel, mndcubesiegel, mndquartsiegel, mndexposiegel,
   mndtrigosiegel, mndexpo, mndtrigo, mndmatesiegel, mndmating, mndsingpert,
   mndherman, mndnewtonsiegel, mndnewton, mndcubicnewton, mndquarticnewton

   These classes are part of Mandel 5.12, which is free software; you can
   redistribute and / or modify them under the terms of the GNU General
   Public License as published by the Free Software Foundation; either
   version 3, or (at your option) any later version. In short: there is
   no warranty of any kind; you must redistribute the source code as well.
*/

#include <cmath>
using namespace std;
#include "mndynamo.h"

void mndynamics::root(double x, double y, double &u, double &v)
{  v = sqrt(x*x + y*y);
   if (x > 0.0) { u = sqrt(0.5*(v + x)); v = 0.5*y/u; return; }
   if (x < 0.0)
   { v = sqrt(0.5*(v - x)); if (y < 0.0) v = -v; u = 0.5*y/v; return; }
   if (y >= 0.0) { u = sqrt(0.5*y); v = u; return; }
   u = sqrt(-0.5*y); v = -u;
}

void mndynamics::iterate(double &x, double &y, int mode) const // = 0
{  if (!mode) f(A, B, x, y);
   if (mode > 0) critical(A, B, x, y);
}

void mndynamics::prepare(int sg, uint nmax, uint &dm, double *t)
{  //set parameters before drawing
   sign = sg; maxiter = nmax; if (maxiter > 65000) maxiter = 65000;
   if (!maxMode(sign)) dm = 0;
   else if (dm > maxMode(sign) && dm <= 15) dm = 1;
   if (dm < minMode(sign)) dm = minMode(sign);
   drawmode = dm; if (drawmode == 1) drawmode = 16;
   if (drawmode > 255) //renormalization
   {  temp[0] = t[0];
      maxiter = nmax - (drawmode & 255); maxiter /= (drawmode >> 8);
      if (sg < 0) //reduce preperiod (from cr.v. to cr.pt.)
      { if (!(drawmode & 255)) drawmode += (drawmode >> 8); drawmode--; }
   }
   if (!drawmode && sign < 0) //cycles
   {  double X = A, Y = B; Period = period(X, Y, 1);
      if (Period > 12 || !Period) X = 1.0 + bailout;
      temp[0] = X; temp[1] = Y;
   }
   else temp[1] = *t;
}

uint mndynamics::period(double &a, double &b, int cycle) // = 0
{  //determine the period, if cycle then set a, b to periodic point.
   uint j; double x, y, x0, y0; critical(a, b, x, y);
   for (j = 1; j <= 1000; j++)
   { if (x*x + y*y <= bailout) f(a, b, x, y); else return 0; }
   x0 = x; y0 = y;
   for (j = 1; j <= 1024; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 0;
      if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < 1e-16)
      {  if (cycle) { a = x; b = y; }
         return j;
      }
   }
   return 10000;
}

uint mndynamics::esctime(double x, double y)
{  uint j; double a, b, x0, y0;
   if (sign > 0) { a = x; b = y; critical(a, b, x, y); }
   else { a = A; b = B; }
   if (drawmode > 255) return renormtime(a, b, x, y);
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return j;
      if (sign < 0 && !drawmode
         && (x - temp[0])*(x - temp[0]) + (y - temp[1])*(y - temp[1]) < 1e-6)
      {  int cl = 1 + j % Period;
         if (Period <= 4 && j % (2*Period) >= Period ) cl |= 8;
         return 65280 | cl;
      }
   }
   if (drawmode) return 65280 | (drawmode & 15);
   if (sign > 0)
   {  x0 = x, y0 = y;
      for (j = 0; j < 60; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y);
         else return maxiter + j;
         if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < .000001 )
           return 65281 + (j  % 12);
      }
   }
   return 65293;
}

uint mndynamics::renormtime(double a, double b, double x, double y)
{  uint j; int i, per = drawmode >> 8, preper = drawmode & 255;
   double c, d; critical(a, b, c, d);
   for (i = 1; i <= preper; i++)
   { if (x*x + y*y <= bailout) f(a, b, x, y); else return 0; }
   for (j = 1; j <= maxiter; j++)
   {  if ( (x - c)*(x - c) + (y - d)*(y - d) > *temp) return j;
      for (i = 1; i <= per; i++)
      { if (x*x + y*y <= bailout) f(a, b, x, y); else return 0; }
   }
   return 65293;
}

uint mndynamics::pixcolor(double x, double y)
{  uint j, cl; double a, b;
   if (sign > 0) { a = x; b = y; critical(a, b, x, y); }
   else { a = A; b = B; }
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y);
      else
      {  if (drawmode == 2)
         { cl = ((j % 24) >> 3); if (!cl) cl = 4; if (j & 1) cl |= 8; }
         else cl = j;
         return cl;
      }
   }
   return 0;
}

////////////////////////////////////////////////////////////////////

void mndsiegel::iterate(double &x, double &y, int mode) const // = 0
{  if (!mode) f(A, B, x, y);
   if (mode <= 0) return;
   which = mode; critical(A, B, x, y);
}

//wrong for persistently attracting or parabolic != 0
uint mndsiegel::period(double &a, double &b, int cycle) // = 0
{  //determine period, may set a, b to per. pt.. Set which to active cr.pt.
   uint j; int point; double x, y, x0, y0; critical(a, b, x, y);
   for (point = 0; point <= 1; point ++)
   {  if (point) { which ^= 3; critical(a, b, x, y); }
      for (j = 1; j <= 1000; j++)
      { if (x*x + y*y <= bailout) f(a, b, x, y); else return 0; }
      x0 = x; y0 = y;
      /*if (x*x + y*y > 0.001)*/ for (j = 1; j <= 1024; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 0;
         if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < 1e-8)
         {  if (cycle) { a = x; b = y; }
            return j;
         }
      }
   }
   return 10000;
}

uint mndsiegel::esctime(double x, double y)
{  if (sign < 0) return mndynamics::esctime(x, y);
   uint j; int point; double x0, y0, a = x, b = y;
   critical(a, b, x, y);
   if (drawmode > 255) return renormtime(a, b, x, y);
   for (point = 0; point <= 1; point++)
   {  if (point) { which ^= 3; critical(a, b, x, y); }
      for (j = 1; j <= maxiter; j++)
      { if (x*x + y*y <= bailout) f(a, b, x, y); else return j; }
      if (point && drawmode) return 65280 | (drawmode & 15);
      x0 = x; y0 = y;
      if (! drawmode /*&& x*x + y*y > 1e-3*/) for (j = 0; j < 60; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y);
         else return maxiter + j;
         if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < 1e-6)
            return 65281 + (j  % 12);
      }
   }
   return 65293;
}

uint mndsiegel::pixcolor(double x, double y)
{  if (sign < 0) return mndynamics::pixcolor(x, y);
   uint j, cl, point; double a = x, b = y;
   critical(a, b, x, y);
   for (point = 0; point <= 1; point++)
   {  if (point) { which ^= 3; critical(a, b, x, y); }
      for (j = 1; j <= maxiter; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y);
         else
         {  if (drawmode == 2)
            { cl = ((j % 24) >> 3); if (!cl) cl = 4; if (j & 1) cl |= 8; }
            else cl = j;
            return cl;
         }
      }
   }
   return 0;
}

////////////////////////////////////////////////////////////////////

void mndcubicsiegel::f(double a, double b, double &x, double &y) const
{  if (subtype == 2) { f2(a, b, x, y); return; }
   double u = x*x - y*y + a*x - b*y + L, v = 2*x*y + a*y + b*x + M,
   w = x*u - y*v;  y = x*v + y*u; x = w;
}  //z^3 + cz^2 + lambda z

void mndcubicsiegel::f2(double a, double b, double &x, double &y) const
{  double rb, ib;
   root(a*a - b*b - a + 0.25*(1 - L), 2*a*b - b - 0.25*M, rb, ib);
   double u = x*x - y*y, v = 2*x*y, X = a*x - b*y + rb, Y = a*y + b*x + ib,
     w = X*u - Y*v - (a + 1)*x + b*y - rb;
   y = X*v + Y*u - (a + 1)*y - b*x - ib; x = w;
}  //cz^3 + bz^2 - (c+1)z -b , b^2 = (c - .5)^2 - lambda/4

void mndcubicsiegel::critical(double a, double b, double &x, double &y) const
{  if (subtype == 1)
   {  root(-a*a + b*b + 3*L, -2*a*b + 3*M, y, x); //factor i
      if (which & 1) x = -x; else y = -y;
      x -= a; y -= b; x /= 3.0; y /= 3.0; return;
   }
   double rb, ib, u, v;
   root(a*a - b*b - a + 0.25*(1 - L), 2*a*b - b - 0.25*M, rb, ib);
   root(4*(a*a - b*b) + 2*a + 0.25*(1 - L), 8*a*b + 2*b - 0.25*M, u, v);
   if (which & 1) { u = -u; v = -v; }
   u -= rb; v -= ib; rb = 3*(a*a + b*b); if (rb < 1e-20) return;
   x = (a*u + b*v)/rb; y = (a*v - b*u)/rb;
}

void mndcubicsiegel::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg) { L = xmid; M = rewidth; return; }
   if (subtype == 1) { xmid = 0; rewidth = 4.2; }
   else { xmid = 0; if (sign > 0) rewidth = 1.5; else rewidth = 2.3; }
}

int mndcubicsiegel::bifurcate(double t, double &a, double &b) const
{  if (subtype != 1) return 0;
   if (t <= -1.0) return 1;
   t *= (2*PI); double l = cos(t) + L - 2.0, m = sin(t) + M, s = b;
   root(l, -m, a, b); t = sqrt(l*l + m*m); if (t <= 0) return 0;
   a /= t; b /= t; l += (L - 1.0); m += M;
   t = a*l - b*m; b = a*m + b*l; a = t; if (s*b < 0) { a = -a; b = -b; }
   return 1;
}

////////////////////////////////////////////////////////////////////

void mndquartsiegel::f(double a, double b, double &x, double &y) const
{  double Ar, Ai, ur, ui;
   if (subtype <= 2) { Ar = a; Ai = b; ur = subtype - 1; ui = 0; }
   if (subtype >= 3) { ur = a; ui = b; }
   if (subtype == 3) { Ar = 3*ur - L; Ai = 3*ui - M; }
   if (subtype == 4)
   {  double r = ur*ur + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      double m = ur/r - 3, n = -ui/r,
      w = 0.25*(L*m - M*n + 6*ur); n = 0.25*(L*n + M*m + 6*ui); m = w;
      r = (ur-0.5)*(ur-0.5) + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      Ar = ((ur - 0.5)*m + ui*n)/r; Ai = ((ur - 0.5)*n - ui*m)/r;
   }
   if (subtype >= 5)
   {  double r = ur*ur + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      Ar = 2 + ((1 - L)*ur - M*ui)/r; Ai = ((L - 1)*ui - M*ur)/r;
   }
   if (subtype == 6)
   {  double m = ur*ur - ui*ui, n = 2*ur*ui,
      r = (ur - 1)*m - ui*n; n = (ur - 1)*n + ui*m; m = r;
      r = m*m + n*n; if (r < 1e-10) { x = 1e100; return; }
      Ar -= m/r; Ai += n/r;
   }
   double u = x*x - y*y, v = 2*x*y, m = L - 2*(Ar+ur), n = M - 2*(Ai+ui),
   w = Ar*u - Ai*v + m*x - n*y + Ar + 3*ur - 2*L;
   n = Ar*v + Ai*u + m*y + n*x + Ai + 3*ui - 2*M; m = w;
   w = m*u - n*v + L*x - M*y; y = m*v + n*u + L*y + M*x; x = w;
}  //Az^4 + (lambda-2A-2u)z^3 + (A+3u-2lambda)z^2 + lambda z

void mndquartsiegel::critical(double a, double b, double &x, double &y) const
{  double Ar, Ai, ur, ui;
   if (subtype <= 2) { Ar = a; Ai = b; ur = subtype - 1; ui = 0; }
   if (subtype >= 3) { ur = a; ui = b; }
   if (subtype == 3) { Ar = 3*ur - L; Ai = 3*ui - M; }
   if (subtype == 4)
   {  double r = ur*ur + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      double m = ur/r - 3, n = -ui/r,
      w = 0.25*(L*m - M*n + 6*ur); n = 0.25*(L*n + M*m + 6*ui); m = w;
      r = (ur-0.5)*(ur-0.5) + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      Ar = ((ur - 0.5)*m + ui*n)/r; Ai = ((ur - 0.5)*n - ui*m)/r;
   }
   if (subtype >= 5)
   {  double r = ur*ur + ui*ui; if (r < 1e-10) { x = 1e100; return; }
      Ar = 2 + ((1 - L)*ur - M*ui)/r; Ai = ((L - 1)*ui - M*ur)/r;
   }
   if (subtype == 6)
   {  double m = ur*ur - ui*ui, n = 2*ur*ui,
      r = (ur - 1)*m - ui*n; n = (ur - 1)*n + ui*m; m = r;
      r = m*m + n*n; if (r < 1e-10) { x = 1e100; return; }
      Ar -= m/r; Ai += n/r;
   }
   double u = 3*ur + Ar - 1.5*L, v = 3*ui + Ai - 1.5*M; Ar *= 4; Ai *= 4;
   root(u*u - v*v + Ar*L - Ai*M, 2*u*v + Ar*M + Ai*L, x, y);
   if (which & 1) {x = -x; y = -y; }
   x += u; y += v;
   double r = Ar*Ar + Ai*Ai; if (r < 1e-10) { x = 1e100; return; }
   u = (Ar*x + Ai*y)/r; y = (Ar*y - Ai*x)/r; x = u;
}

void mndquartsiegel::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg) { L = xmid; M = rewidth; return; }
   if (sg < 0)
   {  xmid = 0;
      if (subtype == 1) rewidth = 1.8;
      if (subtype == 2) rewidth = 1.7;
      if (subtype == 3) rewidth = 2.6;
      if (subtype == 4) rewidth = 2.3;
      if (subtype == 5) rewidth = 1.9;
      if (subtype == 6) rewidth = 2.1;
      return;
   }
   if (subtype == 1) { xmid = 8.2; rewidth = 13.2; }
   if (subtype == 2) { xmid = 1; rewidth = 11.8; }
   if (subtype == 3) { xmid = .42; rewidth = 1.06; }
   if (subtype == 4) { xmid = .3; rewidth = 2; }
   if (subtype == 5) { xmid = .4; rewidth = 2.15; }
   if (subtype == 6) { xmid = .25; rewidth = 2.15; }
}

/////////////////////////////////////////////////////////////////////

void mndexposiegel::f(double a, double b, double &x, double &y) const
{ if (x > 200) { x = 1e100; return; }
  double u = a + L, v = b + M, m = u*x - v*y - a, n = u*y + v*x - b;
  x = exp(x); u = x*cos(y); v = x*sin(y);
  x = m*u - n*v + a; y = m*v + n*u + b;
}

void mndexposiegel::critical(double a, double b, double &x, double &y) const
{  if (which & 1) { x = a; y = b; return; }
   x = L + a; y = M + b;
   a = x*x + y*y; if (a < 1e-10) { x = 1e100; return; }
   b = -(L*x + M*y)/a; y = (L*y - M*x)/a; x = b;
}

void mndexposiegel::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg) { L = xmid; M = rewidth; return; }
   xmid = 0; rewidth = 5.0; if (sg < 0) rewidth = 2.7;
}

//////////////////////////////////////////////////////////////////////

void mndtrigosiegel::f(double a, double b, double &x, double &y) const
{  if (fabs(y) > 200) { x = 1e100; y = 1e100; return; }
   double s = sin(x), c = cos(x), S = sinh(y), C = cosh(y);
   x = C*(L*s - a*c) - S*(M*c + b*s) + a;
   y = C*(M*s - b*c) + S*(L*c + a*s) + b;
}

void mndtrigosiegel::critical(double a, double b, double &x, double &y) const
{  root(a*a - b*b + L*L - M*M, 2*(a*b + L*M), x, y);
   if (which & 1) { x = -x; y = -y; }
   x += a; y += b;
}

void mndtrigosiegel::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg) { L = xmid; M = rewidth; return; }
   xmid = 0; rewidth = 6.78; if (sg < 0) rewidth = 4.84;
}

/////////////////////////////////////////////////////////////////////

void mndexpo::f(double a, double b, double &x, double &y) const
{  if (x > 200) { x = 1e100; return; }
   double w = exp(x), u = w*cos(y), v = w*sin(y);
   if (subtype & 2) { w = a*x - b*y; b = a*y + b*x; a = w; }
   w = a*u - b*v; y = a*v + b*u; x = w;
}

void mndexpo::startPlane(int sg, double &xmid, double &rewidth) const
{  xmid = 0; rewidth = 8.1;
   if (sg < 0 && (subtype & 1) ) rewidth = 4.8;
   if (sg > 0 && (subtype & 2) ) { xmid = 7.1; rewidth = 20.5; }
}

//////////////////////////////////////////////////////////////////////

void mndtrigo::f(double a, double b, double &x, double &y) const
{  double u, v; if (fabs(y) > 200) { x = 1e100; y = 1e100; return; }
   if (subtype == 1)
   {  u = sin(x)*cosh(y); v = cos(x)*sinh(y);
      x = a*u - b*v; y = a*v + b*u; return;
   }
   u = cos(x)*cosh(y); v = -sin(x)*sinh(y);
   if (subtype == 6)
   {  double U = 1 - cos(a)*cosh(b), V = sin(a)*sinh(b), W = U*U + V*V;
      if (W < 1e-10) { x = 1e100; return; }
      u--; x = (u*U + v*V)/W + 1; v = (v*U - u*V)/W; u = x;
      x = a*u - b*v; y = a*v + b*u;
   }
   if (subtype == 3)
   { x = PI*u + a; y = PI*v + b; return; }
   if (subtype == 4) u++;
   if (subtype == 5) u--;
   x = a*u - b*v; y = a*v + b*u;
}

void mndtrigo::startPlane(int sg, double &xmid, double &rewidth) const
{  xmid = 0; rewidth = 4.84;
   if (sg > 0 && subtype == 1) rewidth = 6.0;
   if (sg > 0 && subtype == 4) rewidth = 4.0;
}

//////////////////////////////////////////////////////////////////////

void mndmatesiegel::f(double a, double b, double &x, double &y) const
{  double w = x*x + y*y; if (w < 1e-10) { x = 1e100; return; }
   x += x/w; y -= y/w; w = L*x - M*y + a; y = L*y + M*x + b; x = w;
}// lambda(z + 1/z) + c , lambda -> 1/lambda before

void mndmatesiegel::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg)                                                     //?????
   {  double w = xmid*xmid + rewidth*rewidth;
      if (w > 1e-10) { L = xmid/w; M = -rewidth/w; return; }
   }
   xmid = 0; rewidth = 15.5; if (sg < 0) rewidth = 6.5;
}

//////////////////////////////////////////////////////////////////////

void mndmating::f(double a, double b, double &x, double &y) const
{  if (subtype == 6) { F(a, b, x, y); return; }
   double Ar, Ai, Br = a, Bi = b, aa = a*a - b*b, bb = 2*a*b, N;
   if (subtype == 1)
   { Ar = -0.5*(aa + 1); Ai = -a*b; }
   if (subtype == 2)
   { Ar = a; Ai = b; Br = rb; Bi = ib; }
//   { Ar = a; Ai = b; Br = -a; Bi = -b; }
//   { Ar = a; Ai = b; Br = -2.0 - a; Bi = -b; }
   if (subtype == 3)
   { Ar = aa*a - bb*b - a - 1; Ai = bb*a + aa*b - b; Br = -aa; Bi = -bb; }
   if (subtype == 4)
   { Ar = -a; Ai = -b; }
   if (subtype == 5)
   { Ar = aa*a - bb*b; Ai = aa*b + bb*a; }
   N = x*x - y*y; y = 2*x*y; x = N; Ar += x; Ai += y; Br += x; Bi += y;
   N = Br*Br + Bi*Bi; if (N < 1e-100) { x = 1e100; return; }
   x = (Ar*Br + Ai*Bi)/N; y = (Ai*Br - Ar*Bi)/N;
}// (z^2 + A)/(z^2 + B)

void mndmating::F(double a, double b, double &x, double &y) const
{  double w = x*x + y*y; if (w < 1e-10) { x = 1e100; return; }
   x += (x/w + 2); y -= y/w; w = a*x - b*y + 1; y = a*y + b*x; x = w;
}// c(z + 1/z + 2) + 1

void mndmating::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg && subtype == 2) { rb = xmid; ib = rewidth; return; }
   xmid = 0; rewidth = 3.0;
   if (subtype == 1 && sg > 0) rewidth = 13.0;
   if (subtype == 3 && sg < 0) rewidth = 5.0;
   if (subtype == 4 && sg > 0) { xmid = -1.88; rewidth = 3.75; }
   if (subtype == 4 && sg < 0) rewidth = 7.7;
   if (subtype == 5 && sg > 0) { xmid = 1.0; rewidth = 2.95; }
   if (subtype == 5 && sg < 0) rewidth = 7.5;
   if (subtype == 6 && sg > 0) rewidth = 1.5;
   if (subtype == 6 && sg < 0) rewidth = 8.0;
}

int mndmating::bifurcate(double t, double &a, double & b) const
{  if (subtype > 4) return 0;      return 0;//activate after rewriting
   int per = 1; if (subtype == 1 && (a < -2/3.0 || a > 1/3.0) ) per = 2;
   if (t <= -1.0) return per;
   t *= (2*PI); double l = cos(t), m = sin(t);
/*   t = a; complex c, p, q, mu = complex(l/3, m/3);
   if (subtype == 2)
   { l = -1/48.0; p = (3*mu - 2)*mu/16; q = mu*(mu - 1)*(mu - 1)/8; }
   else
   { l = -0.6; p = (-3*mu*mu + 2*mu - 3)/8; q = -mu*(mu - 1)*(mu - 1)/4; }
   q = p*p - q; root(real(q), imag(q), a, b); q = complex(a, b); p += q;
   if ( (real(p) - l)*(t - l) <= 0) p -= 2*q;
   a = real(p); b = imag(p);//*/
   return 1;
} //bifurcate

uint mndmating::esctime(double x, double y) //escape locus grayscale
{  if (subtype == 6 && sign > 0 && x*x + y*y >= 1.0) return 0;
   if (sign > 0 || drawmode || Period > 6) return mndynamics::esctime(x, y);
   uint j;
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(A, B, x, y);
      else
      {  if (subtype & 1)
         { int cl = 8*((j % 3) - 1); if (cl < 0) cl = 7; return 65280 | cl; }
         if (j & 1) return 65280 | (drawmode & 15); else return 65288;
      }
      if ((x - temp[0])*(x - temp[0]) + (y - temp[1])*(y - temp[1]) < 1e-6)
      {  int cl = 1 + j % Period;
         if (Period <= 4 && j % (2*Period) >= Period ) cl |= 8;
         return 65280 | cl;
      }
   }
   return 65293;
}

uint mndmating::pixcolor(double x, double y)
{  if (drawmode == 4) //subtype == 2
   {  uint cl1;
      if (sign < 0) { cl1 = marty(A, B, x, y); return (cl1 ? cl1 : 10); }
      cl1 = marty(x, y, 0, 0);
      //bifu.locus of 0 red, infinity blue, section magenta, exterior is green
      if (subtype == 2 && (rb + 1.0)*(rb + 1.0) + ib*ib > 1e-20)
      {  uint cl2 = (marty(x, y, 1.0, 0) ? 9 : 0);
         cl1 |= cl2;  if (cl1 == 13) cl1 = 5;
      }
      return (cl1 ? cl1 : 10);
   }
   uint j, cl; double a, b; //subtype == 2 | 3, drawmode == 2
   if (sign > 0) { a = x; b = y; critical(a, b, x, y); }
   else { a = A; b = B; }
   for (j = 1; j <= maxiter; j++)
   {  f(a, b, x, y);
      if (x*x + y*y > bailout)
      {  if (drawmode & 1) return j; // == 3
         if (subtype & 1) // == 3
         { cl = j % 3; if (!cl) cl = 4; return cl + 8*(j & 1); }
         return 1 + (j & 1) + 4*(j & 2);
      }
   }
   return 0;
}

uint mndmating::marty(double a, double b, double x, double y)
{  uint j; double xp = (sign > 0 ? 0 : 1.0), yp = 0, u, v, w;
   a -= rb; b -= ib; // c - b
   for (j = 1; j <= maxiter; j++)
   {  u = x*x - y*y + rb; v = -(2*x*y + ib); w = u*u + v*v;
      if (w > 1e100 || w < 1e-100) return 0;
      u /= w; v /= w; // 1/(z^2 + B)
      w = 2*(x*xp - y*yp); yp = 2*(x*yp + y*xp); xp = w;
      w = b*yp - a*xp; yp = -(a*yp + b*xp); xp = w; // 2(b - c)*z*zp
      x = u*u - v*v; y = 2*u*v;
      w = x*xp - y*yp; yp = x*yp + y*xp; xp= w; // "" /= (z^2 + b)^2
      if (sign > 0) { xp += u; yp += v; }
      x = a*u - b*v + 1.0; y = a*v + b*u;
      u = xp*xp + yp*yp; v = x*x + y*y + 1;
      if (u > 1e100 || v > 1e100) return 0;
      if (temp[1]*u > 1.44*v*v) return 12;
   }
   return 0;
}

//////////////////////////////////////////////////////////////////////

void mndmenage::f(double a, double b, double &x, double &y) const
{  double u = x*x, v = y*y, w = u + v; if (w < 1e-20) { x = 1e100; return; }
   u -= v; v = 2*x*y; w *= w; x += 0.5*u/w; y -= 0.5*v/w;
   w = a*x - b*y; y = a*y + b*x; x = w;
}

void mndmenage::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; if (sg > 0) rewidth = 1.1; else rewidth = 16.0; }

void mndmenage::prepare(int sg, uint nmax, uint &dm, double *t)
{  uint DM = dm; mndynamics::prepare(sg, nmax, dm, t);
   if (DM == 4) { dm = 4; drawmode = 4; temp[1] = *t; } // > maxmode
 }

uint mndmenage::pixcolor(double x, double y)
{ if (sign < 0) return marty(A, B, x, y); else return marty(x, y, 1.0, 0); }

uint mndmenage::marty(double a, double b, double x, double y)
{  uint j; double xp = (sign > 0 ? 0 : 1.0), yp = 0, u, v, w;
/*   a -= rb; b -= ib; // c - b
   for (j = 1; j <= maxiter; j++)
   {  u = x*x - y*y + rb; v = -(2*x*y + ib); w = u*u + v*v;
      if (w > 1e20 || w < 1e-20) return 0;
      u /= w; v /= w; // 1/(z^2 + B)
      w = 2*(x*xp - y*yp); yp = 2*(x*yp + y*xp); xp = w;
      w = b*yp - a*xp; yp = -(a*yp + b*xp); xp = w; // 2(b - c)*z*zp
      x = u*u - v*v; y = 2*u*v;
      w = x*xp - y*yp; yp = x*yp + y*xp; xp= w; // zp /= (z^2 + b)^2
      if (sign > 0) { xp += u; yp += v; }
      x = a*u - b*v + 1.0; y = a*v + b*u;
      u = xp*xp + yp*yp; v = x*x + y*y + 1;
      if (u > 1e60 || v > 1e20) return 1;
      if (temp[1]*u > 1.44*v*v) return 9;
   }//*/
   return 0;
}

//////////////////////////////////////////////////////////////////////

void mndsingpert::f(double a, double b, double &x, double &y) const
{  double u = x*x - y*y; y *= (2*x); x = u;
   u = x*x + y*y; if (u < 1e-20) { x = 1e100; return; }
   a /= u; b /= u;
   u = x + a*x + b*y; y += (b*x - a*y); x = u;
}

void mndsingpert::critical(double a, double b, double &x, double &y) const
{ root(a, b, x, y); root(x, y, x, y); }

void mndsingpert::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; if (sg > 0) rewidth = 0.4; else rewidth = 1.3; }

int mndsingpert::bifurcate(double t, double &a, double & b) const
{  if (t <= -1.0) return 1; t *= (2*PI); double l = cos(t), m = sin(t);
   a = l*l - m*m; b = 2*l*m; l = a + 4*l + 4.0; m = b + 4*m;
   a = 4.0 - a; b = -b; t = a*l - b*m; b = a*m + b*l;
   a = t/256.0; b /= 256.0;
   return 1; // 256*c = (L + 2)^3 * (2 - L)
}

//////////////////////////////////////////////////////////////////////

void mndherman::f(double a, double b, double &x, double &y) const
{  double u = x - a, v = y - b, U = 1 - a*x + b*y, V = -a*y - b*x, r;
   r = U*U + V*V; if (r < 1e-10) { x = 1e100; return; }
   U /= r; V /= r; r = u*U + v*V; v = v*U - u*V; u = r; // u+iv=(z-c)/(1-cz)
   U = x*x - y*y; V = 2*x*y; r = u*U - v*V; v = v*U + u*V; u = r; // *= z^2
   x = u*L - v*M; y = u*M + v*L; // *= (L + iM)
}

void mndherman::critical(double a, double b, double &x, double &y) const
{  double u, v, w = a*a + b*b; if (w < 1e-10) { x = 0.0; y = 0.0; return; }
   w = 0.75/w; x = a*(0.25 + w); y = b*(0.25 - w); // (c^2+3)/(4c)
   root(x*x - y*y - 1.0, 2*x*y, u, v);
   x += u; y += v; // z^2 - (c^2+3)/(2c)z + 1 = 0 .
}

void mndherman::iterate(double &x, double &y, int mode) const // = 0
{  if (!mode) f(A, B, x, y);
   if (mode <= 0) return;
   critical(A, B, x, y); if (mode & 1) return;
   double w = x*x + y*y; if (w < 1e-10) { x = 0.0; y = 0.0; return; }
   x /= w; y = -1.0/w; // product of cr.pt.s is 1
}

void mndherman::startPlane(int sg, double &xmid, double &rewidth) const
{  if (!sg) { L = xmid; M = rewidth; return; }
   if (sg < 0) { xmid = 2.5; rewidth = 8.0; }
   else { xmid = 4.0; rewidth = 3.0; }
}

uint mndherman::pixcolor(double x, double y)
{  uint j; if (sign > 0) return 9;
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y > bailout) return 10;
      if (x*x + y*y < 1e-6) return 12;
      f(A, B, x, y);
   }
   return 0;
}

//////////////////////////////////////////////////////////////////////

void mndnewtonsiegel::f(double a, double b, double &x, double &y) const
{  /*double rp, ip, u, v, w;

   rp = 4*x - 6*a + 6.0; ip = 4*y - 6*b;
   w = x*rp - y*ip - 12*a; ip = x*ip + y*rp - 12*b; rp = w;
   w = x*rp - y*ip + 10*a - 10.0; ip = x*ip + y*rp + 10*b; rp = w; // P'

   u = 3*x - 4*a + 4.0; v = 3*y - 4*b;
   w = x*u - y*v - 6*a; v = x*v + y*u - 6*b; u = w;
   w = x*u - y*v; v = x*v + y*u; u = w;
   w = x*u - y*v + 18*a - 7.0; y = x*v + y*u + 18*b; x = w; // zP' - P

   w = rp*rp + ip*ip; if (w < 1e-40) { x = 1e100; return; }
   a = (x*rp + y*ip)/w; y = (y*rp - x*ip)/w; x = a;*/
}

void mndnewtonsiegel::critical(double a, double b, double &x, double &y) const
{
}

void mndnewtonsiegel::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; rewidth = 2.5; }

uint mndnewtonsiegel::esctime(double x, double y)
{  //if (sign < 0 || drawmode > 255) return mndnewton::esctime(x, y);
   uint j; int point; double x0, y0, a = x, b = y;
   critical(a, b, x, y); x0 = x; y0 = y;
   for (point = 0; point <= 1; point++)
   {  if (point) { which ^= 3; critical(a, b, x, y); x0 = x; y0 = y; }
      for (j = 1; j <= maxiter; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 65280;
         if ( (x - x0)*(x - x0) + (y -y0)*(y -y0) < 1e-5) return j;
         x0 = x, y0 = y;
      }
      if (point && drawmode) return 65280 | (drawmode & 15);
      if (!drawmode) for (j = 0; j < 60; j++)
      {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 65280;
         if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < 1e-6 )
         { if (j) return 65281 + (j  % 12); else break; }
      }
   }
   return 65293;
}

////////////////////////////////////////////////////////////////////

void mndnewtonpara::f(double a, double b, double &x, double &y) const
{  double u = x*x - y*y + 2.0*x - a*a + b*b, v = 2.0*(x*y + y - a*b),
   w = u*u + v*v; if (w < 1.0e-50) { x = 1.0e100; return; }
   w = 2.0/w; u *= w; v *= w;
   w = x + x*u + y*v; y += y*u - x*v; x = w - 1.0;
}  // z - F/F'  with  F = (z^2 - c^2)*e^z

void mndnewtonpara::critical(double a, double b, double &x, double &y) const
{  root(2.0 + a*a - b*b, 2.0*a*b, x, y);
   if (which & 1) { x = -x; y = -y; }
   x -= 2.0;
}

uint mndnewtonpara::singlecolor(double a, double b, double x, double y)
{  uint j;
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 0;
      if (x < -100.0) return 9; //1 + 8*(j & 1);
      if ( (x - a)*(x - a) + (y - b)*(y - b) < 1.0e-6) return 2 + 8*(j & 1);
      if ( (x + a)*(x + a) + (y + b)*(y + b) < 1.0e-6) return 4 + 8*(j & 1);
   }
   return 0;
}

uint mndnewtonpara::pixcolor(double x, double y)
{  if (sign < 0) return singlecolor(A, B, x, y);
   double u, v; critical(x, y, u, v); uint cl = singlecolor(x, y, u, v);
   if (!(cl & 1)) return cl;
   which ^= 3; return singlecolor(x, y, -4.0 - x, -y);
}

void mndnewtonpara::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; rewidth = 5.0; }

////////////////////////////////////////////////////////////////////

void mndnewtonqara::f(double a, double b, double &x, double &y) const
{  double u, v, w, U, V, aa = a*a - b*b + 1.0, bb = 2.0*a*b;
   U = aa*x - bb*y - 2.0*a; V = aa*y + bb*x - 2.0*b; //F
   u = x + a; v = y + b; w = U*u - V*v + aa; v = U*v + V*u + bb; u = w; //F'
   w = u*u + v*v; if (w < 1.0e-50) { x = 1.0e100; return; }
   w = 1.0/w; u *= w; v *= w;
   x -= U*u + V*v; y += U*v - V*u;
}  // z - F/F'  with  F = [(c^2 + 1)z - 2c]*e^{z^2/2 + cz}
//corresponds to  z*e^{z^2/2 + Cz}  with  C = c + 2c/(c^2 + 1)

void mndnewtonqara::critical(double a, double b, double &x, double &y) const
{  if (which == 1) { x = 0.0; y = 0.0; return; }
   double w, aa = a*a - b*b, bb = 2.0*a*b;
   root(bb*bb - aa*aa - 3.0*aa - 3.0, -2.0*aa*bb - 3.0*bb, x, y);
   if (which & 1) { x = -x; y = -y; }
   x += bb*b - aa*a; y -= aa*b + bb*a;
   aa++; w = aa*aa + bb*bb; if (w < 1.0e-50) { x = 1.0e100; return; }
   w = 1.0/w; aa*= w; bb *= w; w = x*aa + y*bb; y = y*aa - x*bb; x = w;
}

uint mndnewtonqara::singlecolor(double a, double b, double x, double y)
{  uint j; double aa = a*a - b*b + 1.0, bb = 2.0*a*b, cc = aa*aa + bb*bb;
   if (cc < 1.0e-20) return 0;
   cc = 2.0/cc; aa *= cc; bb *= cc; cc = aa*a + bb*b; bb = aa*b - bb*a; aa=cc;
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 0;
      if (y > 10.0) return 12;
      if (y < -10.0) return 9;
      if ((x - aa)*(x - aa) + (y - bb)*(y - bb) < 1.0e-6) return 2 + 8*(j & 1);
   }
   return 0;
}

uint mndnewtonqara::pixcolor(double x, double y)
{  if (sign < 0) return singlecolor(A, B, x, y);
   double u, v; critical(x, y, u, v);
   uint cl, cl0 = singlecolor(x, y, u, v); if (!(cl0 & 5)) return cl0;
   which++; if (which > 3) which = 1; critical(x, y, u, v);
   cl = singlecolor(x, y, u, v); if (!(cl & 5)) return cl; else cl0 += cl;
   which++; if (which > 3) which = 1; critical(x, y, u, v);
   cl = singlecolor(x, y, u, v); if (!(cl & 5)) return cl; else cl0 += cl;
   return (cl0 & 1 ? 12 : 9);
}

void mndnewtonqara::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; rewidth = 3.5; }

////////////////////////////////////////////////////////////////////

void mndnewtonrara::f(double a, double b, double &x, double &y) const
{  double u = x*x - y*y + a*x - b*y + 1.0, v = 2.0*x*y + a*y + b*x,
      w = a*a - b*b; b *= 2.0*a; a = w;
   w = u*u + v*v; if (w < 1.0e-50) { x = 1.0e100; return; }
   w = 1.0/w; u *= w; v *= w;
   x -= a*u + b*v; y += a*v - b*u;
}  // z - F/F'  with  F'/F = [z^2 + cz + 1]/c^2

void mndnewtonrara::critical(double a, double b, double &x, double &y) const
{  if (which <= 2) { a = -a; b = -b; }
   root(a - 1.0, b, x, y); if (which & 1) { x = -x; y = -y; }
   if (which >= 3) { x -= a; y -= b; }
} // z^2 + c + 1 = 0  or  (z + c)^2 = c - 1

uint mndnewtonrara::singlecolor(double a, double b, double x, double y)
{  uint j; if (a*a + b*b < 1.0e-50) return 0;
   double X = atan2(b, a), Y = -X/1.5, u = cos(Y), v = sin(Y);
   for (j = 1; j <= maxiter; j++)
   {  X = x*x + y*y; if (X <= bailout) f(a, b, x, y); else return 0;
      if (X < 1000.0) continue; X = u*x - v*y; Y = u*y + v*x;
      if (X < 2.0*Y && X < -2.0*Y) return 9; if (X < 0) continue;
      if (X < Y) return 10;
      if (X < -Y) return 12;
   }
   return 0;
}

uint mndnewtonrara::pixcolor(double x, double y)
{  if (sign < 0) return singlecolor(A, B, x, y);
   double u, v; critical(x, y, u, v);
   uint cl, cl0 = singlecolor(x, y, u, v); if (!cl0) return 0;
   which++; if (which > 4) which = 1; critical(x, y, u, v);
   cl = singlecolor(x, y, u, v); if (!cl) return 0; else cl0 += cl;
   which++; if (which > 4) which = 1; critical(x, y, u, v);
   cl = singlecolor(x, y, u, v); if (!cl) return 0; else cl0 += cl;
   which++; if (which > 4) which = 1; critical(x, y, u, v);
   return ((singlecolor(x, y, u, v) + cl0 + 1) & 15); // cl + cl0 -31
}

void mndnewtonrara::startPlane(int sg, double &xmid, double &rewidth) const
{  if (sg > 0) { xmid = 12.0; rewidth = 8.0; }
   else { xmid = 0; rewidth = 20.0; }
}

//////////////////////////////////////////////////////////////////////

uint mndnewton::esctime(double x, double y)
{  uint j; double a, b, x0, y0;
   if (sign > 0) { a = x; b = y; critical(a, b, x, y); }
   else { a = A; b = B; }
   if (drawmode > 255) return renormtime(a, b, x, y);
   x0 = x; y0 = y;
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 65280;
      if ( (x - x0)*(x - x0) + (y -y0)*(y -y0) < 1e-5) return j;
      if (sign < 0 && !drawmode && Period > 1
         && (x - temp[0])*(x - temp[0]) + (y - temp[1])*(y - temp[1]) < 1e-6)
      {  int cl = 1 + j % Period;
         if (Period <= 4 && j % (2*Period) >= Period ) cl |= 8;
         return 65280 | cl;
      }
      x0 = x, y0 = y;
   }
   if (drawmode) return 65280 | (drawmode & 15);
   if (sign > 0) for (j = 0; j < 60; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 65280;
      if ( (x - x0)*(x - x0) + (y - y0)*(y - y0) < 1e-6 )
      { if (j) return 65281 + (j  % 12); else break; }
   }
   return 65293;
}

uint mndnewton::pixcolor(double x, double y)
{  uint j, l; double a, b;
   if (sign > 0) { a = x; b = y; critical(a, b, x, y); }
   else { a = A; b = B; }
   fixed(a, b); //sets temp[] to fixed points
   for (j = 1; j <= maxiter; j++)
   {  if (x*x + y*y <= bailout) f(a, b, x, y); else return 0;
      for (l = 0; l < degree; l++) if( (x - temp[2*l])*(x - temp[2*l])
           + (y - temp[2*l+1])*(y - temp[2*l+1]) < 1e-6)
      { if (l == 4) l = 5; if (!l) l = 4; return (l + 8*(j & 1)); }
   }
   return 0;
}

//////////////////////////////////////////////////////////////////////

void mndcubicnewton::f(double a, double b, double &x, double &y) const
{  double w = a*a - b*b + .75; b = 2*a*b; a = w; // c = lambda^2 + 3/4
   double u = 2*(x*a - y*b) + 3*(1 - a), v = 2*(x*b + y*a) - 3*b; //numerator
   a = 3*(x*x - y*y) - a; b = 6*x*y - b; //denominator
   w = u*a + v*b; v = v*a - u*b; u = w;
   w = a*a + b*b; if (w < 1e-100) return;
   x = (2*x + u/w)/3.0; y = (2*y + v/w)/3.0;
}

void mndcubicnewton::fixed(double a, double b) const
{  temp[0] = -0.5 + a; temp[1] = b;
   temp[2] = 1.0; temp[3] = 0;
   temp[4] = -0.5 - a; temp[5] = -b;
}

void mndcubicnewton::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; rewidth = 2.0; if (sg < 0) rewidth = 2.5; }

//////////////////////////////////////////////////////////////////////

void mndquarticnewton::f(double a, double b, double &x, double &y) const
{  if (subtype == 2) { f2(a, b, x, y); return; }
   if (subtype == 8) { f8(a, b, x, y); return; }
   double rp, ip, rq, iq, u, v, w = a*a + b*b; if (w < 1e-20) return;
   if (subtype == 1)
   {  rp = (a - a/w)/sqrt(2.0); ip = (b + b/w)/sqrt(2.0);
      rq = 1.5*(rp*rp - ip*ip) + 1.0; iq = 3*rp*ip;
   }
   if (subtype == 4)
   {  rp = 3*a + 1.0; ip = 3*b;
      rq = (4*(rp*rp - ip*ip) -2*rp + 1.0)/3.0; iq = (8*rp*ip - 2*ip)/3.0;
   }
   if (subtype == 6)
   {  rp = a + a/w; ip = b - b/w;
      rq = 7.0 - sqrt(12.0)*(b + b/w); iq = sqrt(12.0)*(a - a/w);
   }
   if (subtype == 7)
   {  rp = a + a/w - 3.0; ip = b - b/w;
      rq = -6*rp - 5.0; iq = -6*ip;
   }

   u = 3*x - 4*rp; v = 3*y - 4*ip;
   w = x*u - y*v + rq - 1.0; v = x*v + y*u + iq; u = w;
   a = x*x - y*y; b = 2*x*y;
   w = a*u - b*v + rq; b = a*v + b*u  + iq; a = w; // zP' - P

   u = 4*x - 6*rp; v = 4*y - 6*ip;
   w = x*u - y*v + 2*rq - 2.0; v = x*v + y*u + 2*iq; u = w;
   rp = x*u - y*v + 2*rp; ip = x*v + y*u + 2*ip; // P'

   w = rp*rp + ip*ip; if (w < 1e-40) { x = 1e100; return; }
   x = (a*rp + b*ip)/w; y = (b*rp - a*ip)/w;
}

void mndquarticnewton::f2(double a, double b, double &x, double &y) const
{  double rp, ip, u, v, w;
   w = a*a - b*b; b *= (2*a); a = w;
   u = x*x - y*y; v = 2*x*y;

   rp = 4*u - 12*a; ip = 4*v - 12*b;
   w = x*rp - y*ip; ip = x*ip + y*rp; rp = w; // P'

   x = 3*u - 6*a; y = 3*v - 6*b;
   w = x*u - y*v; y = x*v + y*u; x = w;
   w = a*a - b*b - 2.25; b *= (2*a); a = w;
   x += (a*a - b*b); y += (2*a*b); // zP' - P

   w = rp*rp + ip*ip; if (w < 1e-40) { x = 1e100; return; }
   a = (x*rp + y*ip)/w; y = (y*rp - x*ip)/w; x = a;
}

void mndquarticnewton::f8(double a, double b, double &x, double &y) const
{  double rp, ip, u, v, w;

   rp = 4*x - 6*a + 6.0; ip = 4*y - 6*b;
   w = x*rp - y*ip - 12*a; ip = x*ip + y*rp - 12*b; rp = w;
   w = x*rp - y*ip + 10*a - 10.0; ip = x*ip + y*rp + 10*b; rp = w; // P'

   u = 3*x - 4*a + 4.0; v = 3*y - 4*b;
   w = x*u - y*v - 6*a; v = x*v + y*u - 6*b; u = w;
   w = x*u - y*v; v = x*v + y*u; u = w;
   w = x*u - y*v + 18*a - 7.0; y = x*v + y*u + 18*b; x = w; // zP' - P

   w = rp*rp + ip*ip; if (w < 1e-40) { x = 1e100; return; }
   a = (x*rp + y*ip)/w; y = (y*rp - x*ip)/w; x = a;
}

void mndquarticnewton::fixed(double a, double b) const
{  if (subtype == 2)
   {  double aa = a*a - b*b; b *= (2*a); a = aa;
      temp[0] = a + 1.5; temp[1] = b;
      temp[4] = -a - 1.5; temp[5] = -b;
      temp[2] = -b; temp[3] = a - 1.5;
      temp[6] = b; temp[7] = -a + 1.5; return;
   }
   double w = a*a + b*b; if (w < 1e-20) return;
   temp[0] = -1.0; temp[1] = 0;
   temp[4] = 1.0; temp[5] = 0;
   if (subtype == 1)
   {  temp[2] = (a - a/w)/sqrt(2.0) - 0.5*(b - b/w);
      temp[3] = (b + b/w)/sqrt(2.0) + 0.5*(a + a/w);
      temp[6] = (a - a/w)/sqrt(2.0) + 0.5*(b - b/w);
      temp[7] = (b + b/w)/sqrt(2.0) - 0.5*(a + a/w);
   }
   if (subtype == 4)
   {  temp[2] = 3*a + 1.0 - sqrt(3.0)*b; temp[3] = 3*b + sqrt(3.0)*a;
      temp[6] = 3*a + 1.0 + sqrt(3.0)*b; temp[7] = 3*b - sqrt(3.0)*a;
   }
   if (subtype == 6)
   {  temp[2] = 2*a; temp[3] = 2*b - sqrt(3.0);
      temp[6] = 2*a/w; temp[7] = -2*b/w + sqrt(3.0);
   }
   if (subtype == 7)
   {  temp[2] = 2*a - 3.0; temp[3] = 2*b;
      temp[6] = 2*a/w - 3.0; temp[7] = -2*b/w;
   }
}

void mndquarticnewton::critical(double a, double b, double &x, double &y) const
{  double w = a*a + b*b; if (w < 1e-20) return;
   if (subtype == 1)
   { x = (a - a/w)/sqrt(8.0); y = (b + b/w)/sqrt(8.0); }
   if (subtype == 2 || subtype == 4 || subtype == 8)
   { x = a; y = b; }
   if (subtype == 6)
   {  double rph = 0.5*(a + a/w), iph = 0.5*(b - b/w); //rp/2, ip/2
      x = 1.0 - (b + b/w)/sqrt(3.); y = (a - a/w)/sqrt(3.0); //(rq - 1, iq)/6
      root(rph*rph - iph*iph - x, 2*rph*iph - y, x, y);
      x += rph; y += iph;
   }
   if (subtype == 7)
   { x = a + a/w - 2.0; y = b - b/w; }

}

void mndquarticnewton::startPlane(int sg, double &xmid, double &rewidth) const
{ xmid = 0; rewidth = 2.5; }

//////////////////////////////////////////////////////////////////////
